from sqlmodel import SQLModel
from sqlalchemy import and_, or_
from typing import Any

class Filter:
    def __init__(self):
        pass

    def _eq(self, entity:SQLModel, column_name, value):
        q = getattr(entity, column_name) == value
        return q

    def _gt(self, entity:SQLModel, column_name, value):
        q = getattr(entity, column_name) > value
        return q
    
    def _gte(self, entity:SQLModel, column_name, value):
        q = getattr(entity, column_name) >= value
        return q
    
    def _ilike(self, entity:SQLModel, column_name, value):
        q = getattr(entity, column_name).ilike(value)
        return q

    def _in(self, entity:SQLModel, column_name, value):
        if not isinstance(value, list):
            raise ValueError(f"Value {value} is not a list")
        q = getattr(entity, column_name).in_(value)
        return q
    
    def _iregex(self, entity:SQLModel, column_name, value):
        raise NotImplementedError
    
    def _regex(self, entity:SQLModel, column_name, value):
        raise NotImplementedError
    
    def _is_null(self, entity:SQLModel, column_name, value):
        q = getattr(entity, column_name) == None
        return q
    
    def _like(self, entity:SQLModel, column_name, value):
        q = getattr(entity, column_name).like(value)
        return q
    
    def _lt(self, entity:SQLModel, column_name, value):
        q = getattr(entity, column_name) < value
        return q
    
    def _lte(self, entity:SQLModel, column_name, value):
        q = getattr(entity, column_name) <= value
        return q
    
    def _similar(self, entity:SQLModel, column_name, value):
        raise NotImplementedError
    
    def _neq(self, entity:SQLModel, column_name, value):
        return getattr(entity, column_name) != value
    
    def _nilike(self, entity:SQLModel, column_name, value):
        q = ~getattr(entity, column_name).ilike(value)
        return q
    
    def _nin(self, entity:SQLModel, column_name, value):
        if not isinstance(value, list):
            raise ValueError(f"Value {value} is not a list")
        q = ~getattr(entity, column_name).in_(value)
        return q
    
    def _niregex(self, entity:SQLModel, column_name, value):
        raise NotImplementedError
    
    def _nlike(self, entity:SQLModel, column_name, value):
        q = ~getattr(entity, column_name).like(value)
        return q
    
    def _nregex(self, entity:SQLModel, column_name, value):
        raise NotImplementedError
    
    def _nsimilar(self, entity:SQLModel, column_name, value):
        raise NotImplementedError

    def _or(self, entity, args: list[dict[str, Any]]):
        q = or_(*(self.process_where(entity, arg) for arg in args))
        return q

    def _and(self, entity, args: list[dict[str, Any]]):
        q = and_(*(self.process_where(entity, arg) for arg in args))
        return q

    def process_where(self, entity: SQLModel, where: dict[str, Any] | list[dict[str, Any]]) -> None:
        for column_name, column_value in where.items():
            if isinstance(column_value, list):
                return getattr(self, column_name)(entity, column_value)
            for operator, value in column_value.items():
                return getattr(self, operator)(entity, column_name, value)