from sqlalchemy import and_, or_
from sqlmodel import SQLModel

from typing import Any

class DynamicQueryFilters:
    def __init__(self, entity: SQLModel):
        self.entity = entity
        self.where = []

    def _eq(self, column_name, value):
        q = getattr(self.entity, column_name) == value
        return q

    def _or(self, args: list[dict[str, Any]]):
        q = or_(*(self.process_where(arg) for arg in args))
        return q

    def _and(self, args: list[dict[str, Any]]):
        q = and_(*(self.process_where(arg) for arg in args))
        # print(f"and : {q}, {args}")
        return q

    def process_where(self, where: dict[str, Any] | list[dict[str, Any]]) -> None:
        for column_name, column_value in where.items():
            if isinstance(column_value, list):
                return getattr(self, column_name)(column_value)
            for operator, value in column_value.items():
                return getattr(self, operator)(column_name, value)
